import logging
import numpy as np
import matplotlib.pyplot as pl
from revrand import classification, basis_functions
from sklearn.linear_model import LogisticRegression
from sklearn.preprocessing import PolynomialFeatures
import scipy.stats as stats

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


def normaliseInputs(trainingX):
    Xmean = np.mean(trainingX, axis=0)
    X_norm = trainingX - Xmean
    Xmin = np.min(X_norm, axis=0); Xmax = np.max(X_norm, axis=0)
    Xrange = Xmax-Xmin
    X_norm = X_norm/Xrange
    return X_norm, Xmean, Xrange

#collate and normalise data
print('Loading data...')
data_path = '/home/socallaghan/data/seafloor/'
data = np.load(data_path + 'seafloorData_final_flipped.npz')
lat = data['lat']
lon = data['lon']
flipped = True
if flipped == True:
    lat = data['lon']
    lon = data['lat']
labels = data['labels'].ravel().astype(int)
ndata = len(labels)


# A la Carte classifier setting
nbases = 2500
lenscale = 0.1
reg = 1000.

# method = 'SGD'
method = 'SGD'
batchsize = 100
rate = 0.9
eta = 1e-6
passes = 10
# lenscale_array = np.logspace(-3,0,9)
# lenscale_array = np.logspace(-1.875,-1.125,9)
xlen = np.logspace(-1.875,-1.125,3)
ylen = np.logspace(-1.875,-1.125,3)
lenscale_array = [np.array([x,y]) for x in xlen for y in ylen]

#Pad the boundaries
lonPos170180Ind = (lon>170)*(lon<180)
lonNeg170180Ind = (lon<-170)*(lon>-180)

padPosLon = lon[lonNeg170180Ind]+360
padPosLat = lat[lonNeg170180Ind]
padPosLabel = labels[lonNeg170180Ind[:,0]]

padNegLon = lon[lonPos170180Ind]-360
padNegLat = lat[lonPos170180Ind]
padNegLabel = labels[lonPos170180Ind[:,0]]

lon = np.append(np.append(lon,padPosLon), padNegLon)
lat = np.append(np.append(lat, padPosLat), padNegLat)
labels = np.append(np.append(labels, padPosLabel), padNegLabel)

trainingPoints = np.array([lon,lat]).T

# Create test data
# xeva, yeva = np.meshgrid(np.linspace(-180,180,3601),np.linspace(-90,90,1801))
xeva, yeva = np.meshgrid(np.linspace(-180,180,361),np.linspace(-90,90,181))

xeva = xeva.T
yeva = yeva.T
testPoints = np.c_[xeva.ravel(), yeva.ravel()]

X, Xmean, Xrange = normaliseInputs(trainingPoints)
Xs = (testPoints - Xmean)/Xrange
Y = labels

# Plot scatter
Phi = basis_functions.RandomRBF_ARD(nbases, X.shape[1])
pl.figure()


if method == 'SGD':

    weights_array=[]
    fun_array = []
     # cross val for loop
    for i, lenscale in enumerate(lenscale_array):
        pl.subplot(3,3,i+1)
        weights, labels, fun = classification.learn_sgd(X, Y, Phi, tuple(lenscale),
                                               regulariser=reg, eta=eta,
                                             batchsize=batchsize, rate=rate,
                                             passes=passes)
        weights_array.append(weights)
        fun_array.append(fun)
        optimum_idx = np.argmin(fun_array)
        Ey = classification.predict(Xs, weights, Phi, tuple(lenscale))
        Ey_argmax = np.argmax(Ey,axis=1)
        pl.scatter(Xs[:,0],Xs[:,1],c=Ey_argmax,linewidths=0,s=50)

elif method == 'MAP':
    weights, labels = classification.learn_map(X, Y, Phi, (lenscale,),
                                               regulariser=reg)
else:
    raise ValueError("Invalid method chosen!")
pl.show()
lreg = LogisticRegression(penalty='l2')
lreg.fit(X, Y)

pl.scatter(Xs[:,0],Xs[:,1],c=stats.entropy(Ey.T),linewidths=0,s=50)
pl.plot(X[:,0],X[:,1],'b.')
pl.show()

# Predict at test points
# Predict
# pys_l = classification.predict(Xs, weights, Phi, tuple(lenscale_array))
Ey = classification.predict(Xs, weights_array[optimum_idx], Phi, tuple(lenscale_array[optimum_idx]))
Ey_argmax = np.argmax(Ey,axis=1)
pl.scatter(Xs[:,0],Xs[:,1],c=Ey_argmax,linewidths=0,s=50); pl.show()
# plot results
# pl.imshow(np.reshape(Ey[:,0],xeva.shape)); pl.show()


