# Import the relevant packages
import os
import pandas as pd
import numpy as np
from sklearn.svm import LinearSVC
from sklearn.svm import SVC
from sklearn.feature_selection import RFECV
from sklearn.cross_validation import StratifiedKFold
from sklearn import cross_validation
from matplotlib import pyplot as pl
import IPython


# Load the data
currentpath = os.getcwd()
data = pd.read_csv('seafloor_lith_data_all_features.csv')
for i in range(len(data.columns)): # Remove any nans
  data  = data[np.isfinite(data[data.columns[i]])]

# Select the rock label for analysis
rockLabel = 7
print(rockLabel)
nIterations = 3

# Set up the features (X) and the targets or labels (y) for the classifier
X = data[data.columns[3:]].values
y = data.lithology.values
print('Shape of X:', X.shape)
print('removing rocks from northern hemosphere')
southHemId = data[data.columns[1]].values<0
X = np.delete(X,np.arange(X.shape[0])*southHemId,0)
y = np.delete(y,np.arange(y.shape[0])*southHemId,0)

#Log certain features,
X[:,4] = np.log(X[:,4])
X[:,6] = np.log(X[:,6])

# Adding random features to X
nRandomFeatures = 1
X = np.append(X,np.random.random([X.shape[0],nRandomFeatures]),axis=1)

#normalised data:,
means = np.mean(X,axis=0)
std_devs = np.std(X,axis=0)
X = (X - means)/std_devs

XOrig = X
yOrig = y

nFeatures = X.shape[1]
# Initialise results matrix
featureFreq = np.zeros([nFeatures-nRandomFeatures,nFeatures])

# We repeat the experiment multiple times to average over
# the randomness introduced by the cross validation sets being randomly chosen
for iter in range(nIterations):
    print('Rock Label: ' + str(rockLabel))
    print('Iteration: '+ str(iter+1) + ' of ' + str(nIterations))

    #One vs. All,
    X = XOrig.copy()
    y = yOrig.copy()
    y[y!=rockLabel] = 0

    #Cull classes to balance the dataset,
    Xtest = X.copy()
    yTest = y.copy()
    maxNum = 1500
    minNum = 500

    nPosSamples = np.sum(yTest==rockLabel)
    if nPosSamples < maxNum :
        maxNum = nPosSamples
        minNum = nPosSamples

    for i in (np.arange(int(np.max(y))+1)):

      # Remove classes with less than minimum amount,
      if np.sum(yTest==i)<minNum:
          #print(np.sum(y==i))
          Xtest = Xtest[yTest!=i,:]
          yTest = yTest[yTest!=i]

      # Subsample classes with higher than maximum amount,
      if np.sum(yTest==i)>maxNum:
          index = np.arange(yTest.shape[0])[yTest==i]
          np.random.shuffle(index)
          Xtest = np.delete(Xtest,index[maxNum:],axis=0)
          yTest = np.delete(yTest,index[maxNum:])

    X = Xtest.copy()
    y = yTest.copy()

    # Split data in training and testing,
    X_train, X_test, y_train, y_test = cross_validation.train_test_split(
          X, y, test_size=0.3, random_state=0)

      # Test accuracy of every permutation of features:,
    binaryFeatures = 2**nFeatures

    scoreList = []
     # Generate an empty array where each row will shortly be populated by
     # binary variables indicating whether feature was used or not
    featOnListArray = np.zeros([binaryFeatures-1,nFeatures])
      # Loop over all permutations,
    for i in range(binaryFeatures-1):
          print('Permutation number: '+str(i) + ' of ' + str(binaryFeatures-1))

          # Generating X matrix of selected features,
          featOnStr = list(bin(i+1))
          del featOnStr[1]
          featOnList = [(int(featOnStr[i])) for i in range(len(featOnStr))]
          padding = [0]*(nFeatures-len(featOnList))
          padding.extend(featOnList)
          featOnList = padding[::-1][:nFeatures]
          featOnListArray[i,:] = featOnList
          featOnList = np.nonzero(featOnList)[0]
          X_Train_Selected = X_train[:,featOnList]
          X_Test_Selected = X_test[:,featOnList]

          #Evaluate accuracy using these features,
          svc = SVC(kernel='rbf', probability=True,C=10,gamma=0.1)
          svc.fit(X_Train_Selected,y_train)
          scoreList.append(svc.score(X_Test_Selected, y_test))

      # Plot sorted features (highest score on top)
    maxFeatSortedID = np.argsort(scoreList)[::-1]
    maxFeatSorted = featOnListArray[maxFeatSortedID,:]

    for i in range(8):
          indexBoole = np.sum(maxFeatSorted,axis=1)==i
          maxFeatSorted[indexBoole,:]

    for nFeaturesIter in (np.arange(8)+1):
          indexBoole = np.sum(maxFeatSorted,axis=1)==nFeaturesIter
          featureFreq[nFeaturesIter-1,:] = featureFreq[nFeaturesIter-1,:] + \
                                           maxFeatSorted[indexBoole,:][0,:]
          np.savetxt('featureFreq0'+ str(rockLabel)+'Northern.txt',featureFreq)

#Average over the number of times the experiment was run
featureFreq = featureFreq/(np.sum(featureFreq,axis=1)[0])

np.savetxt('featureFreq0'+ str(rockLabel)+'NorthernHem.txt',featureFreq)


# Plot the Feature Usefulness for the chosen Rock Label
pl.figure()
pl.imshow(featureFreq,interpolation='nearest',extent=(0.5,9.5,8.5,0.5))
pl.xlabel('Feature ID')
pl.ylabel('Number of Features')
pl.colorbar()
ax = pl.gca()
ax.tick_params(axis='x', top='off')
ax.tick_params(axis='y', right='off')
ax.get_yaxis().set_tick_params(direction='out')
ax.get_xaxis().set_tick_params(direction='out')
pl.savefig('multvarAnalysisLabel'+str(rockLabel)+'NorthernHem.pdf')
pl.show()

