# Python multivariate analysis (Support Vector Classification) script written by Simon O'Callaghan NICTA
# 14 May 2015
#

import os
import pandas as pd
import numpy as np

# Import the relevant packages for machine learning tasks 
from sklearn.svm import LinearSVC
from sklearn.svm import SVC
from sklearn.feature_selection import RFECV
from sklearn.cross_validation import StratifiedKFold
from sklearn import cross_validation
from matplotlib import pyplot as pl
import utils
# Import the data, remove NaNs and get absolute values (required for normalizing)
currentpath = os.getcwd()
data = pd.read_csv('seafloor_lith_data_all_features.csv')
for i in range(len(data.columns)):
  data  = data[np.isfinite(data[data.columns[i]])]  # Remove any nans
  data = np.absolute(data)    # Get absolute values
  
# Set up the features (X) and the targets or labels (y) 

# Choices for labels: Ridgetype or Tier
# X data must exclude classifier columns (here listed in column zero of data)

X = data[data.columns[3:]].values
y = data.lithology.values
print('Shape of X:', X.shape)
#print('removing rocks from northern hemisphere')
#southHemId = data[data.columns[1]].values<0
#X = np.delete(X,np.arange(X.shape[0])*southHemId,0)
#y = np.delete(y,np.arange(y.shape[0])*southHemId,0)


# Set the subtype that will be tested for ad number of iterations
ridgeLabel = 5
print(ridgeLabel)
maxSampPerClass = 5000

# All columns of data need to be within same order of magnitude before normalizing and conducting the SVC
# For this purpose the log of cloumns that have values > 100

#Log certain features,
X[:,4] = np.log(X[:,4])
X[:,6] = np.log(X[:,6])

# Add an artificial feature (filled with random numbers) to assess whether the classification is meaningful.
# If the artificial feature is resolved in the analysis, then the other features are less likely to be meaningful.
# Adding random features to X
nRandomFeatures = 1
X = np.append(X,np.random.random([X.shape[0],nRandomFeatures]),axis=1)

# Data needs to be normalized (expressed as a ratio to 1, so that they have the same weighting in analysis)

#normalised data:,
means = np.mean(X,axis=0)
std_devs = np.std(X,axis=0)
X = (X - means)/std_devs

# Initialise results matrix

print('Ridge Label: ' + str(ridgeLabel))

#One vs. All,
y[y!=ridgeLabel] = 0
y[y==ridgeLabel] = 1

# Split data in training and testing,
for i in range(100):

    XCull, yCull = utils.cullData(X,y,maxSampPerClass)

    X_train, X_test, y_train, y_test = cross_validation.train_test_split(
          XCull, yCull, test_size=0.1)

    #Evaluate accuracy using these features,
    # svc = SVC(kernel='rbf', probability=True,C=10,gamma=0.1)
    # svc = SVC(kernel='rbf')
    # svc.fit(X_Train_Selected,y_train)
    # scoreList.append(svc.score(X_Test_Selected, y_test))

    lin_clf = LinearSVC(penalty='l1', dual=False)
    lin_clf.fit(X_train,y_train)
    lin_clf.score(X_test, y_test)

    print(lin_clf.coef_.ravel())
    np.argsort(np.abs(lin_clf.coef_.ravel()))
    pl.plot(np.abs(lin_clf.coef_.ravel()))

pl.show()

