__author__ = 'socallaghan'

import numpy as np
from sklearn.svm import SVC
from sklearn import cross_validation

def genFeatureArray(nFeatures):
        binaryFeatures = 2**nFeatures - 1
        featOnArray = np.zeros([binaryFeatures, nFeatures])
        for i in range(binaryFeatures):
          print('Permutation number: '+str(i) + ' of ' + str(binaryFeatures-1))

          # Generating X matrix of selected features,
          featOnStr = list(bin(i+1))
          del featOnStr[1]
          featOnList = [(int(featOnStr[i])) for i in range(len(featOnStr))]
          padding = [0]*(nFeatures-len(featOnList))
          padding.extend(featOnList)
          featOnList = padding[::-1][:nFeatures]
          featOnArray[i,:] = featOnList
          # featOnList = np.nonzero(featOnList)[0]

        return featOnArray


def cullData(X,y,maxSampPerClass):

    XCull = X.copy()
    yCull = y.copy()

    for i in [0, 1]:
        if np.sum(y == i)>maxSampPerClass:
              index = np.arange(yCull.shape[0])[yCull==i]
              np.random.shuffle(index)
              XCull = np.delete(XCull, index[maxSampPerClass:], axis=0)
              yCull = np.delete(yCull, index[maxSampPerClass:])
    return XCull, yCull

def balanceData(X,y):
    '''
    assumes spacing is uniform between labels
    '''

    xBal = X.copy()
    yBal = y.copy()

    nPerGroup, bins = np.histogram(yBal, np.unique(yBal).shape[0])
    maxSampPerClass = np.min(nPerGroup)

    for i in np.unique(yBal):
        if np.sum(y == i)>maxSampPerClass:
              index = np.arange(yBal.shape[0])[yBal==i]
              np.random.shuffle(index)
              xBal = np.delete(xBal, index[maxSampPerClass:], axis=0)
              yBal = np.delete(yBal, index[maxSampPerClass:])
    return xBal, yBal

def evalFeatures(argsIn):

    print("Random Seed")
    np.random.seed()

    X = argsIn[0]
    y = argsIn[1]
    featOnArray = argsIn[2]

    if len(argsIn) > 3:
        maxSampPerClass = argsIn[3]
        XCull, yCull = cullData(X, y, maxSampPerClass)
    else:
        XCull = X.copy(); yCull = y.copy()

    # nFeatures = X.shape[1]
    # binaryFeatures = 2**nFeatures

    X_train, X_test, y_train, y_test = cross_validation.train_test_split(
          XCull, yCull, test_size=0.2, random_state=np.round(np.random.random()*1e5).astype(int))
    scoreList = []
     # Generate an empty array where each row will shortly be populated by
     # binary variables indicating whether feature was used or not
    # featOnArray = np.zeros([binaryFeatures-1,nFeatures])

      # Loop over all permutations,
    for featureSetID in range(np.round(featOnArray.shape[0]).astype(int)):
          if np.mod(featureSetID,1) == 0:
            print('Permutation number: '+str(featureSetID) + ' of ' + str(featOnArray.shape[0]))

          # Generating X matrix of selected features,
          featOnList = np.nonzero(featOnArray[featureSetID,:])[0]
          X_Train_Selected = X_train[:,featOnList]
          X_Test_Selected = X_test[:,featOnList]

          #Evaluate accuracy using these features,
          svc = SVC(kernel='rbf', probability=True)
          svc.fit(X_Train_Selected,y_train)
          scoreList.append(svc.score(X_Test_Selected, y_test))

    return scoreList

def evalFeaturesGP(argsIn):

    print("Random Seed")
    np.random.seed()

    X = argsIn[0]
    y = argsIn[1]
    featOnArray = argsIn[2]

    if len(argsIn) > 3:
        maxSampPerClass = argsIn[3]
        XCull, yCull = cullData(X, y, maxSampPerClass)
    else:
        XCull = X.copy(); yCull = y.copy()

    # nFeatures = X.shape[1]
    # binaryFeatures = 2**nFeatures

    X_train, X_test, y_train, y_test = cross_validation.train_test_split(
          XCull, yCull, test_size=0.2, random_state=np.round(np.random.random()*1e5).astype(int))
    scoreList = []
     # Generate an empty array where each row will shortly be populated by
     # binary variables indicating whether feature was used or not
    # featOnArray = np.zeros([binaryFeatures-1,nFeatures])

      # Loop over all permutations,
    for featureSetID in range(np.round(featOnArray.shape[0]).astype(int)):
            if np.mod(featureSetID,1) == 0:
                print('Permutation number: '+str(featureSetID) + ' of ' + str(featOnArray.shape[0]))

            # Generating X matrix of selected features,
            featOnList = np.nonzero(featOnArray[featureSetID,:])[0]
            X_Train_Selected = X_train[:,featOnList]
            X_Test_Selected = X_test[:,featOnList]

            #Evaluate accuracy using these features,
            from computers import gp

            def kerneldef(h, k):
                return h(1e-3, 1e5, 1) * k('matern3on2', np.asarray([h(5e-2, 1e1, 1)
                                                  for _ in range(X_Train_Selected.shape[1])]))
          # Intialise constants
            WALLTIME = 1000.0
            N_CORES = 1

            # Modeling options
            WHITEFN = 'pca'
            APPROX_METHOD = 'laplace'       # 'laplace' or 'pls'
            MULTI_METHOD = 'OVA'            # 'AVA' or 'OVA'
            FUSE_METHOD = 'EXCLUSION'       # 'MODE' or 'EXCLUSION'
            RESPONSE_NAME = 'probit'        # 'probit' or 'logistic'

            optimiser_config = gp.OptConfig()
            optimiser_config.sigma = gp.auto_range(kerneldef)
            optimiser_config.walltime = WALLTIME
            responsefunction = gp.classifier.responses.get(RESPONSE_NAME)


            learned_classifier = gp.classifier.learn(X_Train_Selected, y_train, kerneldef,
                responsefunction, optimiser_config,
                multimethod=MULTI_METHOD, approxmethod=APPROX_METHOD,
                train=True, ftol=1e-6, processes=N_CORES)

            print_function = gp.describer(kerneldef)
            gp.classifier.utils.print_learned_kernels(print_function, learned_classifier, np.unique(y))

            import logging
            # Print the matrix of learned classifier hyperparameters
            gp.classifier.utils.print_hyperparam_matrix(learned_classifier)

            # predictor_plt = gp.classifier.query(learned_classifier, X)


            predictor_plt = gp.classifier.query(learned_classifier, X_Test_Selected)
            exp_plt = gp.classifier.expectance(learned_classifier, predictor_plt)
            var_plt = gp.classifier.variance(learned_classifier, predictor_plt)
            yq_lde_plt = gp.classifier.linearised_model_differential_entropy(exp_plt, var_plt,
                learned_classifier)
            yq_sd_plt = gp.classifier.equivalent_standard_deviation(yq_lde_plt)
            yq_prob_plt = gp.classifier.predict_from_latent(exp_plt, var_plt,
                learned_classifier, fusemethod=FUSE_METHOD)


            import sklearn.metrics as metrics
            fpr, tpr, thresholds = metrics.roc_curve(y_test, yq_prob_plt)
            roc_auc = metrics.auc(fpr, tpr)
            print(roc_auc)
            scoreList.append(roc_auc)


    return scoreList